"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// packages/@code-like-a-carpenter/foundation-plugin-cloudformation/src/plugin.ts
var plugin_exports = {};
__export(plugin_exports, {
  addToSchema: () => import_foundation_intermediate_representation.schema,
  plugin: () => plugin
});
module.exports = __toCommonJS(plugin_exports);
var import_assert = __toESM(require("assert"));
var import_promises = require("node:fs/promises");
var import_node_path = __toESM(require("node:path"));
var import_js_yaml = __toESM(require("js-yaml"));
var import_js_yaml_cloudformation_schema = require("js-yaml-cloudformation-schema");
var import_foundation_parser = require("@code-like-a-carpenter/foundation-parser");
var import_graphql_codegen_helpers = require("@code-like-a-carpenter/graphql-codegen-helpers");
var import_cdc = require("./cdc/index.cjs");
var import_reducer = require("./cdc/reducer.cjs");
var import_config = require("./config.cjs");
var import_combine_fragments = require("./fragments/combine-fragments.cjs");
var import_table = require("./table.cjs");
var import_transforms = require("./transforms.cjs");
var import_foundation_intermediate_representation = require("@code-like-a-carpenter/foundation-intermediate-representation");
async function getInitialTemplate({
  sourceTemplate
}) {
  if (sourceTemplate) {
    const raw = await (0, import_promises.readFile)(sourceTemplate, "utf8");
    try {
      return JSON.parse(raw);
    } catch {
      return import_js_yaml.default.load(raw, {
        schema: import_js_yaml_cloudformation_schema.CLOUDFORMATION_SCHEMA
      });
    }
  }
  return {
    AWSTemplateFormatVersion: "2010-09-09",
    Resources: {},
    Transform: "AWS::Serverless-2016-10-31"
  };
}
var plugin = (0, import_graphql_codegen_helpers.makePlugin)(
  import_config.ConfigSchema,
  async (schema2, documents, config, info) => {
    const outputFile = info?.outputFile;
    (0, import_assert.default)(outputFile, "outputFile is required");
    const ir = (0, import_foundation_parser.parse)(schema2, documents, config, info);
    const { models, tables } = ir;
    const stacks = /* @__PURE__ */ new Map();
    const allResources = (0, import_combine_fragments.combineFragments)(
      ...tables.map((table) => {
        if (table.hasCdc) {
          const { fragment, stack } = (0, import_cdc.defineTableCdc)(table, config);
          stacks.set("dispatcher.yml", stack);
          return (0, import_combine_fragments.combineFragments)(fragment, (0, import_table.defineTable)(table));
        }
        return (0, import_table.defineTable)(table);
      }),
      ...models.flatMap(
        (model) => model.changeDataCaptureConfig.map((cdc) => {
          const { type } = cdc;
          if (type === "ENRICHER") {
            const { fragment, stack } = (0, import_cdc.defineEnricher)(config, model, cdc);
            stacks.set(cdc.nestedStackLocation, stack);
            return fragment;
          } else if (type === "REACTOR") {
            const { fragment, stack } = (0, import_cdc.defineReactor)(config, model, cdc);
            stacks.set(cdc.nestedStackLocation, stack);
            return fragment;
          } else if (type === "REDUCER") {
            const { fragment, stack } = (0, import_reducer.defineReducer)(config, model, cdc);
            stacks.set(cdc.nestedStackLocation, stack);
            return fragment;
          }
          throw new Error(`Unexpected CDC type ${type}`);
        })
      )
    );
    const initialTemplate = await getInitialTemplate(config);
    const tpl = (0, import_combine_fragments.combineFragments)(
      initialTemplate,
      allResources,
      {
        Globals: {
          Function: {
            Handler: "index.handler",
            MemorySize: 256,
            Runtime: "nodejs18.x",
            Timeout: 30,
            Tracing: "Active"
          }
        },
        Parameters: {
          StageName: {
            AllowedValues: ["development", "production", "test"],
            Default: "development",
            Description: "The name of the stage",
            Type: "String"
          }
        },
        Resources: {}
        // Transform: 'AWS::LanguageExtensions',
      }
    );
    await (0, import_transforms.applyTransforms)(config, ir, tpl, stacks);
    const outDir = import_node_path.default.dirname(outputFile);
    for (const [filename, stack] of stacks) {
      await writeNestedTemplate(config, import_node_path.default.join(outDir, filename), stack);
    }
    return formatTemplate(config, tpl);
  }
);
function formatTemplate(config, tpl) {
  const { format } = config.outputConfig;
  if (format === "json") {
    return JSON.stringify(tpl, null, 2);
  }
  return import_js_yaml.default.dump(tpl, config.outputConfig.yamlConfig);
}
async function writeNestedTemplate(config, outputFile, tpl) {
  await (0, import_promises.mkdir)(import_node_path.default.dirname(outputFile), { recursive: true });
  await (0, import_promises.writeFile)(outputFile, formatTemplate(config, tpl));
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  addToSchema,
  plugin
});
//# sourceMappingURL=plugin.cjs.map
