"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// packages/@code-like-a-carpenter/foundation-plugin-cloudformation/src/table.ts
var table_exports = {};
__export(table_exports, {
  defineTable: () => defineTable
});
module.exports = __toCommonJS(table_exports);
function defineTable({
  enablePointInTimeRecovery,
  enableStreaming,
  hasTtl,
  tableName,
  primaryKey: { isComposite },
  secondaryIndexes
}) {
  const attributeDefinitions = isComposite ? expandCompositeAttribute("pk", "sk") : expandSimpleAttribute("pk");
  const keySchema = isComposite ? expandCompositeKey("pk", "sk") : expandSimpleKey("pk");
  const globalSecondaryIndexes = [];
  const localSecondaryIndexes = [];
  for (const index of secondaryIndexes) {
    if (index.type === "gsi") {
      attributeDefinitions.push(
        ...index.isComposite ? expandCompositeAttribute(index.partitionKeyName, index.sortKeyName) : expandSimpleAttribute(index.partitionKeyName)
      );
      const gsiKeySchema = index.isComposite ? expandCompositeKey(index.partitionKeyName, index.sortKeyName) : expandSimpleKey(index.partitionKeyName);
      globalSecondaryIndexes.push({
        IndexName: index.name,
        KeySchema: gsiKeySchema,
        Projection: {
          ProjectionType: index.projectionType.toUpperCase()
        }
      });
    } else if (index.type === "lsi") {
      attributeDefinitions.push(...expandSimpleAttribute(index.sortKeyName));
      const lsiKeySchema = expandCompositeKey(
        index.partitionKeyName,
        index.sortKeyName
      );
      localSecondaryIndexes.push({
        IndexName: index.name,
        KeySchema: lsiKeySchema,
        Projection: {
          ProjectionType: index.projectionType.toUpperCase()
        }
      });
    }
  }
  const resource = {
    Properties: {
      AttributeDefinitions: attributeDefinitions,
      BillingMode: "PAY_PER_REQUEST",
      ...globalSecondaryIndexes.length ? { GlobalSecondaryIndexes: globalSecondaryIndexes } : {},
      KeySchema: keySchema,
      ...localSecondaryIndexes.length ? { LocalSecondaryIndexes: localSecondaryIndexes } : {},
      SSESpecification: {
        // @ts-expect-error typedef doesn't include intrinsic functions
        SSEEnabled: { "Fn::If": ["IsProd", true, false] }
      },
      Tags: [
        {
          Key: "StageName",
          // @ts-expect-error typedef doesn't include intrinsic functions
          Value: { Ref: "StageName" }
        },
        {
          Key: "TableName",
          Value: tableName
        }
      ],
      ...conditionalObject(enableStreaming, {
        StreamSpecification: {
          StreamViewType: "NEW_AND_OLD_IMAGES"
        }
      }),
      ...conditionalObject(enablePointInTimeRecovery, {
        PointInTimeRecoverySpecification: {
          PointInTimeRecoveryEnabled: { "Fn::If": ["IsProd", true, false] }
        }
      }),
      ...conditionalObject(hasTtl, {
        TimeToLiveSpecification: {
          AttributeName: "ttl",
          Enabled: true
        }
      })
    },
    Type: "AWS::DynamoDB::Table"
  };
  return {
    Conditions: {
      IsProd: { "Fn::Equals": [{ Ref: "StageName" }, "production"] }
    },
    Outputs: {
      [tableName]: {
        Description: `The name of the DynamoDB table for ${tableName}`,
        Export: {
          Name: { "Fn::Sub": `\${AWS::StackName}-${tableName}` }
        },
        Value: { Ref: tableName }
      }
    },
    Resources: {
      [tableName]: resource
    }
  };
}
function expandCompositeAttribute(pk, sk) {
  return [
    { AttributeName: pk, AttributeType: "S" },
    { AttributeName: sk, AttributeType: "S" }
  ];
}
function expandSimpleAttribute(pk) {
  return [{ AttributeName: pk, AttributeType: "S" }];
}
function expandCompositeKey(pk, sk) {
  return [
    { AttributeName: pk, KeyType: "HASH" },
    { AttributeName: sk, KeyType: "RANGE" }
  ];
}
function expandSimpleKey(pk) {
  return [{ AttributeName: pk, KeyType: "HASH" }];
}
function conditionalObject(condition, object) {
  return condition ? object : {};
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  defineTable
});
//# sourceMappingURL=table.cjs.map
