// packages/@code-like-a-carpenter/foundation-plugin-cloudformation/src/stacks/dispatcher.ts
import { combineFragments } from "../fragments/combine-fragments.mjs";
import { makeLogGroup } from "../fragments/log-group.mjs";
function isDispatcherStack(resourceName) {
  return resourceName.startsWith("FNDNSTD");
}
function makeDispatcherStackName(table) {
  return `FNDNSTD${table.dispatcherConfig.functionName}`;
}
function makeDispatcherStack(config) {
  const functionName = "Function";
  const fn = {
    Metadata: {
      BuildMethod: "esbuild",
      buildProperties: {
        EntryPoints: ["./index"],
        External: config.buildProperties.external,
        Minify: config.buildProperties.minify,
        Sourcemap: config.buildProperties.sourcemap,
        Target: config.buildProperties.target
      }
    },
    Properties: {
      // @ts-expect-error - typedef does not know about intrinsic functions
      CodeUri: { Ref: "CodeUri" },
      Events: {
        Stream: {
          Properties: {
            BatchSize: { Ref: "BatchSize" },
            FunctionResponseTypes: ["ReportBatchItemFailures"],
            MaximumRetryAttempts: { Ref: "MaximumRetryAttempts" },
            StartingPosition: "TRIM_HORIZON",
            Stream: { Ref: "StreamArn" }
          },
          Type: "DynamoDB"
        }
      },
      Handler: "index.handler",
      MemorySize: { Ref: "MemorySize" },
      Policies: [
        "AWSLambdaBasicExecutionRole",
        "AWSLambda_ReadOnlyAccess",
        "AWSXrayWriteOnlyAccess",
        "CloudWatchLambdaInsightsExecutionRolePolicy",
        { CloudWatchPutMetricPolicy: {} },
        {
          EventBridgePutEventsPolicy: {
            EventBusName: { Ref: "EventBus" }
          }
        }
      ],
      Runtime: "nodejs18.x",
      Timeout: { Ref: "Timeout" },
      Tracing: "Active"
    },
    Type: "AWS::Serverless::Function"
  };
  return combineFragments(makeLogGroup({ functionName }), {
    AWSTemplateFormatVersion: "2010-09-09",
    Outputs: {
      FunctionName: {
        Export: {
          // eslint-disable-next-line no-template-curly-in-string
          Name: { "Fn::Sub": "${AWS::StackName}-FunctionName" }
        },
        Value: { Ref: "Function" }
      }
    },
    Parameters: {
      BatchSize: {
        Type: "Number"
      },
      CodeUri: {
        Type: "String"
      },
      EventBus: {
        Type: "String"
      },
      MaximumRetryAttempts: {
        Type: "Number"
      },
      MemorySize: {
        Type: "Number"
      },
      StreamArn: {
        Type: "String"
      },
      Timeout: {
        Type: "Number"
      }
    },
    Resources: {
      [functionName]: fn
    },
    Transform: "AWS::Serverless-2016-10-31"
  });
}
export {
  isDispatcherStack,
  makeDispatcherStack,
  makeDispatcherStackName
};
//# sourceMappingURL=dispatcher.mjs.map
