// packages/@code-like-a-carpenter/foundation-plugin-cloudformation/src/template-transforms/globals.ts
import { assert } from "@code-like-a-carpenter/assert";
import { isDispatcherStack, isHandlerStack } from "../stacks/index.mjs";
var globalsTransform = (template, nestedTemplates) => {
  Object.entries(template.Resources).filter(
    ([resourceName, resource]) => resource.Type === "AWS::Serverless::Application" && (isDispatcherStack(resourceName) || isHandlerStack(resourceName))
  ).forEach(([, resource]) => {
    assert(
      resource.Type === "AWS::Serverless::Application",
      "Resource must be a Serverless Application"
    );
    const refs = findRefs(template.Globals ?? {});
    Object.values(nestedTemplates).forEach((tpl) => {
      tpl.Parameters = tpl.Parameters ?? {};
      for (const ref of refs) {
        tpl.Parameters[ref] = {
          Type: "String"
        };
      }
      tpl.Globals = template.Globals;
    });
    assert(resource.Properties, "Resource must have Properties");
    resource.Properties.Parameters = resource.Properties.Parameters ?? {};
    const Parameters = resource.Properties.Parameters;
    for (const ref of refs) {
      Parameters[ref] = `\${${ref}}`;
    }
  });
};
function findRefs(obj) {
  return Object.entries(obj).reduce((acc, [key, value]) => {
    if (typeof value === "object" && value !== null && !Array.isArray(value)) {
      return acc.concat(findRefs(value));
    }
    if (key === "Ref") {
      assert(typeof value === "string", "Ref value must be a string");
      return acc.concat([value]);
    }
    return acc;
  }, []);
}
export {
  globalsTransform
};
//# sourceMappingURL=globals.mjs.map
